@@ -1,26 +1,37 @@
 package File::Slurp;
 
-my $printed ;
+use 5.6.2 ;
 
 use strict;
+use warnings ;
 
 use Carp ;
 use Exporter ;
 use Fcntl qw( :DEFAULT ) ;
 use POSIX qw( :fcntl_h ) ;
-use Symbol ;
-use UNIVERSAL ;
+#use Symbol ;
 
-use vars qw( @ISA %EXPORT_TAGS @EXPORT_OK $VERSION @EXPORT ) ;
+use vars qw( @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION ) ;
 @ISA = qw( Exporter ) ;
 
-%EXPORT_TAGS = ( 'all' => [
-	qw( read_file write_file overwrite_file append_file read_dir ) ] ) ;
+$VERSION = '9999.17';
 
+@EXPORT_OK = qw(
+	slurp
+	prepend_file
+	edit_file
+	edit_file_lines
+) ;
+
+%EXPORT_TAGS = ( 'all' => [ qw(
+	read_file
+	write_file
+	overwrite_file
+	append_file
+	read_dir ),
+	@EXPORT_OK	
+] ) ;
 @EXPORT = ( @{ $EXPORT_TAGS{'all'} } );
-@EXPORT_OK = qw( slurp prepend_file ) ;
-
-$VERSION = '9999.16';
 
 my $max_fast_slurp_size = 1024 * 100 ;
 
@@ -82,28 +93,31 @@ sub read_file {
 	my $file_name = shift ;
 	my $opts = ( ref $_[0] eq 'HASH' ) ? shift : { @_ } ;
 
-	if ( !ref $file_name && 0 &&
-	     -e $file_name && -s _ < $max_fast_slurp_size && ! %{$opts} && !wantarray ) {
+# this is the optimized read_file for shorter files.
+# the test for -s > 0 is to allow pseudo files to be read with the
+# regular loop since they return a size of 0.
 
-		local( *FH ) ;
+	if ( !ref $file_name && -e $file_name && -s _ > 0 &&
+	     -s _ < $max_fast_slurp_size && !%{$opts} && !wantarray ) {
 
-		unless( open( FH, $file_name ) ) {
+
+		my $fh ;
+		unless( sysopen( $fh, $file_name, O_RDONLY ) ) {
 
 			@_ = ( $opts, "read_file '$file_name' - sysopen: $!");
 			goto &_error ;
 		}
 
-		my $read_cnt = sysread( FH, my $buf, -s _ ) ;
+		my $read_cnt = sysread( $fh, my $buf, -s _ ) ;
 
 		unless ( defined $read_cnt ) {
 
-# handle the read error
-
 			@_ = ( $opts,
 				"read_file '$file_name' - small sysread: $!");
 			goto &_error ;
 		}
 
+		$buf =~ s/\015\012/\n/g if $is_win32 ;
 		return $buf ;
 	}
 
@@ -158,9 +172,8 @@ sub read_file {
 
 #printf "RD: BINARY %x MODE %x\n", O_BINARY, $mode ;
 
-# open the file and handle any error
-
-		$read_fh = gensym ;
+		$read_fh = local( *FH ) ;
+#		$read_fh = gensym ;
 		unless ( sysopen( $read_fh, $file_name, $mode ) ) {
 			@_ = ( $opts, "read_file '$file_name' - sysopen: $!");
 			goto &_error ;
@@ -176,7 +189,6 @@ sub read_file {
 
 #print "SIZE $size_left\n" ;
 
-
 # we need a blk_size if the size is 0 so we can handle pseudofiles like in
 # /proc. these show as 0 size but have data to be slurped.
 
@@ -187,24 +199,6 @@ sub read_file {
 		}
 	}
 
-
-# 	if ( $size_left < 10000 && keys %{$opts} == 0 && !wantarray ) {
-
-# #print "OPT\n" and $printed++ unless $printed ;
-
-# 		my $read_cnt = sysread( $read_fh, my $buf, $size_left ) ;
-
-# 		unless ( defined $read_cnt ) {
-
-# # handle the read error
-
-# 			@_ = ( $opts, "read_file '$file_name' - small2 sysread: $!");
-# 			goto &_error ;
-# 		}
-
-# 		return $buf ;
-# 	}
-
 # infinite read loop. we exit when we are done slurping
 
 	while( 1 ) {
@@ -216,8 +210,6 @@ sub read_file {
 
 		unless ( defined $read_cnt ) {
 
-# handle the read error
-
 			@_ = ( $opts, "read_file '$file_name' - loop sysread: $!");
 			goto &_error ;
 		}
@@ -240,9 +232,6 @@ sub read_file {
 
 	${$buf_ref} =~ s/\015\012/\n/g if $is_win32 && !$opts->{'binmode'} ;
 
-# this is the 5 returns in a row. each handles one possible
-# combination of caller context and requested return type
-
 	my $sep = $/ ;
 	$sep = '\n\n+' if defined $sep && $sep eq '' ;
 
@@ -250,19 +239,19 @@ sub read_file {
 
 	if( wantarray || $opts->{'array_ref'} ) {
 
-		my @parts = split m/($sep)/, ${$buf_ref}, -1;
+		use re 'taint' ;
 
-		my @lines ;
+		my @lines = length(${$buf_ref}) ?
+			${$buf_ref} =~ /(.*?$sep|.+)/sg : () ;
 
-		while( @parts > 2 ) {
+		chomp @lines if $opts->{'chomp'} ;
 
-			my( $line, $sep ) = splice( @parts, 0, 2 ) ;
-			push @lines, "$line$sep" ;
-		}
-
-		push @lines, shift @parts if @parts && length $parts[0] ;
+# caller wants an array ref
 
 		return \@lines if $opts->{'array_ref'} ;
+
+# caller wants list of lines
+
 		return @lines ;
 	}
 
@@ -277,37 +266,8 @@ sub read_file {
 # caller passed in an i/o buffer by reference (normal void context)
 
 	return ;
-
-
-# # caller wants to get an array ref of lines
-
-# # this split doesn't work since it tries to use variable length lookbehind
-# # the m// line works.
-# #	return [ split( m|(?<=$sep)|, ${$buf_ref} ) ] if $opts->{'array_ref'}  ;
-# 	return [ length(${$buf_ref}) ? ${$buf_ref} =~ /(.*?$sep|.+)/sg : () ]
-# 		if $opts->{'array_ref'}  ;
-
-# # caller wants a list of lines (normal list context)
-
-# # same problem with this split as before.
-# #	return split( m|(?<=$sep)|, ${$buf_ref} ) if wantarray ;
-# 	return length(${$buf_ref}) ? ${$buf_ref} =~ /(.*?$sep|.+)/sg : ()
-# 		if wantarray ;
-
-# # caller wants a scalar ref to the slurped text
-
-# 	return $buf_ref if $opts->{'scalar_ref'} ;
-
-# # caller wants a scalar with the slurped text (normal scalar context)
-
-# 	return ${$buf_ref} if defined wantarray ;
-
-# # caller passed in an i/o buffer by reference (normal void context)
-
-# 	return ;
 }
 
-
 # errors in this sub are returned as scalar refs
 # a normal IO/GLOB handle is an empty return
 # an overloaded object returns its stringified as a scalarfilename
@@ -482,7 +442,8 @@ sub write_file {
 
 # open the file and handle any error.
 
-		$write_fh = gensym ;
+		$write_fh = local( *FH ) ;
+#		$write_fh = gensym ;
 		unless ( sysopen( $write_fh, $file_name, $mode, $perms ) ) {
 
 			@_ = ( $opts, "write_file '$file_name' - sysopen: $!");
@@ -496,7 +457,6 @@ sub write_file {
 
 	sysseek( $write_fh, 0, SEEK_END ) if $opts->{'append'} ;
 
-
 #print 'WR before data ', unpack( 'H*', ${$buf_ref}), "\n" ;
 
 # fix up newline to write cr/lf if this is a windows text file
@@ -527,12 +487,11 @@ sub write_file {
 
 		unless ( defined $write_cnt ) {
 
-# the write failed
 			@_ = ( $opts, "write_file '$file_name' - syswrite: $!");
 			goto &_error ;
 		}
 
-# track much left to write and where to write from in the buffer
+# track how much left to write and where to write from in the buffer
 
 		$size_left -= $write_cnt ;
 		$offset += $write_cnt ;
@@ -590,8 +549,6 @@ sub append_file {
 	goto &write_file
 }
 
-# basic wrapper around opendir/readdir
-
 # prepend data to the beginning of a file
 
 sub prepend_file {
@@ -615,16 +572,11 @@ sub prepend_file {
 
 #print "PRE [$prepend_data]\n" ;
 
-
-###### set croak as error_mode
-###### wrap in eval
-
 	my $err_mode = delete $opts->{err_mode} ;
 	$opts->{ err_mode } = 'croak' ;
 	$opts->{ scalar_ref } = 1 ;
 
-	my $existing_data ;
-	eval { $existing_data = read_file( $file_name, $opts ) } ;
+	my $existing_data = eval { read_file( $file_name, $opts ) } ;
 
 	if ( $@ ) {
 
@@ -635,11 +587,10 @@ sub prepend_file {
 
 #print "EXIST [$$existing_data]\n" ;
 
-	$opts->{ atomic } = 1 ;
-
-	my $write_result = eval { 
-		write_file( $file_name, $opts,
-			$prepend_data, $$existing_data ) ;
+	$opts->{atomic} = 1 ;
+	my $write_result =
+		eval { write_file( $file_name, $opts,
+		       $prepend_data, $$existing_data ) ;
 	} ;
 
 	if ( $@ ) {
@@ -652,6 +603,116 @@ sub prepend_file {
 	return $write_result ;
 }
 
+# edit a file as a scalar in $_
+
+sub edit_file(&$;$) {
+
+	my( $edit_code, $file_name, $opts ) = @_ ;
+	$opts = {} unless ref $opts eq 'HASH' ;
+
+# 	my $edit_code = shift ;
+# 	my $file_name = shift ;
+# 	my $opts = ( ref $_[0] eq 'HASH' ) ? shift : {} ;
+
+#print "FILE $file_name\n" ;
+
+# delete unsupported options
+
+	my @bad_opts =
+		grep $_ ne 'err_mode' && $_ ne 'binmode', keys %{$opts} ;
+
+	delete @{$opts}{@bad_opts} ;
+
+# keep the user err_mode and force croaking on internal errors
+
+	my $err_mode = delete $opts->{err_mode} ;
+	$opts->{ err_mode } = 'croak' ;
+
+# get a scalar ref for speed and slurp the file into a scalar
+
+	$opts->{ scalar_ref } = 1 ;
+	my $existing_data = eval { read_file( $file_name, $opts ) } ;
+
+	if ( $@ ) {
+
+		@_ = ( { err_mode => $err_mode },
+			"edit_file '$file_name' - read_file: $!" ) ;
+		goto &_error ;
+	}
+
+#print "EXIST [$$existing_data]\n" ;
+
+	my( $edited_data ) = map { $edit_code->(); $_ } $$existing_data ;
+
+	$opts->{atomic} = 1 ;
+	my $write_result =
+		eval { write_file( $file_name, $opts, $edited_data ) } ;
+
+	if ( $@ ) {
+
+		@_ = ( { err_mode => $err_mode },
+			"edit_file '$file_name' - write_file: $!" ) ;
+		goto &_error ;
+	}
+
+	return $write_result ;
+}
+
+sub edit_file_lines(&$;$) {
+
+	my( $edit_code, $file_name, $opts ) = @_ ;
+	$opts = {} unless ref $opts eq 'HASH' ;
+
+# 	my $edit_code = shift ;
+# 	my $file_name = shift ;
+# 	my $opts = ( ref $_[0] eq 'HASH' ) ? shift : {} ;
+
+#print "FILE $file_name\n" ;
+
+# delete unsupported options
+
+	my @bad_opts =
+		grep $_ ne 'err_mode' && $_ ne 'binmode', keys %{$opts} ;
+
+	delete @{$opts}{@bad_opts} ;
+
+# keep the user err_mode and force croaking on internal errors
+
+	my $err_mode = delete $opts->{err_mode} ;
+	$opts->{ err_mode } = 'croak' ;
+
+# get an array ref for speed and slurp the file into lines
+
+	$opts->{ array_ref } = 1 ;
+	my $existing_data = eval { read_file( $file_name, $opts ) } ;
+
+	if ( $@ ) {
+
+		@_ = ( { err_mode => $err_mode },
+			"edit_file_lines '$file_name' - read_file: $!" ) ;
+		goto &_error ;
+	}
+
+#print "EXIST [$$existing_data]\n" ;
+
+	my @edited_data = map { $edit_code->(); $_ } @$existing_data ;
+
+	$opts->{atomic} = 1 ;
+	my $write_result =
+		eval { write_file( $file_name, $opts, @edited_data ) } ;
+
+	if ( $@ ) {
+
+		@_ = ( { err_mode => $err_mode },
+			"edit_file_lines '$file_name' - write_file: $!" ) ;
+		goto &_error ;
+	}
+
+	return $write_result ;
+}
+
+# basic wrapper around opendir/readdir
+
 sub read_dir {
 
 	my $dir = shift ;
@@ -674,6 +735,11 @@ sub read_dir {
 	@dir_entries = grep( $_ ne "." && $_ ne "..", @dir_entries )
 		unless $opts->{'keep_dot_dot'} ;
 
+	if ( $opts->{'prefix'} ) {
+
+		substr( $_, 0, 0, "$dir/" ) for @dir_entries ;
+	}
+
 	return @dir_entries if wantarray ;
 	return \@dir_entries ;
 }
@@ -720,36 +786,39 @@ __END__
 
 =head1 NAME
 
-File::Slurp - Simple and Efficient Reading/Writing of Complete Files
+File::Slurp - Simple and Efficient Reading/Writing/Modifying of Complete Files
 
 =head1 SYNOPSIS
 
   use File::Slurp;
 
 # read in a whole file into a scalar
-
   my $text = read_file( 'filename' ) ;
 
 # read in a whole file into an array of lines
-
   my @lines = read_file( 'filename' ) ;
 
 # write out a whole file from a scalar
-
   write_file( 'filename', $text ) ;
 
 # write out a whole file from an array of lines
-
   write_file( 'filename', @lines ) ;
 
 # Here is a simple and fast way to load and save a simple config file
 # made of key=value lines.
-
   my %conf = read_file( $file_name ) =~ /^(\w+)=(\.*)$/mg ;
   write_file( $file_name, {atomic => 1}, map "$_=$conf{$_}\n", keys %conf ;
 
-# read in a whole directory of file names (skipping . and ..)
+# insert text at the beginning of a file
+  prepend_file( 'filename', $text ) ;
 
+# in-place edit to replace all 'foo' with 'bar' in file 
+  edit_file { s/foo/bar/g } 'filename' ;
+
+# in-place edit to delete all lines with 'foo' from file
+  edit_file_lines sub { $_ = '' if /foo/ }, 'filename' ;
+
+# read in a whole directory of file names (skipping . and ..)
   my @files = read_dir( '/path/to/dir' ) ;
 
 =head1 DESCRIPTION
@@ -803,6 +872,12 @@ impossible to get with a clean read_file call which means you can check
 the return value and always know if you had an error. You can change how
 errors are handled with the C<err_mode> option.
 
+Speed Note: If you call read_file and just get a scalar return value
+it is now optimized to handle shorter files. This is only used if no
+options are used, the file is shorter then 100k bytes, the filename is
+a plain scalar and a scalar file is returned. If you want the fastest
+slurping, use the C<buf_ref> or C<scalar_ref> options (see below)
+
 NOTE: as of version 9999.06, read_file works correctly on the C<DATA>
 handle. It used to need a sysseek workaround but that is now handled
 when needed by the module itself.
@@ -831,13 +906,20 @@ slurped file. The following two calls are equivalent:
 	my $lines_ref = read_file( $bin_file, array_ref => 1 ) ;
 	my $lines_ref = [ read_file( $bin_file ) ] ;
 
+=head3 chomp
+
+If this boolean option is set, the lines are chomped. This only
+happens if you are slurping in a list context or using the
+C<array_ref> option.
+
 =head3 scalar_ref
 
-If this boolean option is set, the return value (only in scalar context)
-will be an scalar reference to a string which is the contents of the
-slurped file. This will usually be faster than returning the plain
-scalar. It will also save memory as it will not make a copy of the file
-to return.
+If this boolean option is set, the return value (only in scalar
+context) will be an scalar reference to a string which is the contents
+of the slurped file. This will usually be faster than returning the
+plain scalar. It will also save memory as it will not make a copy of
+the file to return. Run the extras/slurp_bench.pl script to see speed
+comparisons.
 
 	my $text_ref = read_file( $bin_file, scalar_ref => 1 ) ;
 
@@ -846,7 +928,10 @@ to return.
 You can use this option to pass in a scalar reference and the slurped
 file contents will be stored in the scalar. This can be used in
 conjunction with any of the other options. This saves an extra copy of
-the slurped file and can lower ram usage vs returning the file.
+the slurped file and can lower ram usage vs returning the file. It is
+usually the fastest way to read a file into a scalar. Run the
+extras/slurp_bench.pl script to see speed comparisons.
+
 
 	read_file( $bin_file, buf_ref => \$buffer ) ;
 
@@ -1031,6 +1116,47 @@ explicitly.
 	prepend_file( $file, \@lines ) ;
 	prepend_file( $file, { binmode => 'raw:'}, $bin_data ) ;
 
+
+=head2 edit_file, edit_file_lines
+
+These subs read in a file into $_, execute a code block which should
+modify $_ and then write $_ back to the file. The difference between
+them is that C<edit_file> reads the whole file into $_ and calls the
+code block one time. With C<edit_file_lines> each line is read into $_
+and the code is called for each line. In both cases the code should
+modify $_ if desired and it will be written back out. These subs are
+the equivilent of the -pi command line options of Perl but you can
+call them from inside your program and not fork out a process.
+
+The first argument to C<edit_file> and C<edit_file_lines> is a code
+block or a code reference. The code block is not followed by a comma
+(as with grep and map) but a code reference is followed by a
+comma. See the examples below for both styles. The next argument is
+the filename. The last argument is an optional hash reference and it
+contains key/values that can modify the behavior of
+C<prepend_file>. 
+
+Only the C<binmode> and C<err_mode> options are supported. The
+C<write_file> call has the C<atomic> option set so you will always
+have a consistant file. See above for more about those options.
+
+Each group of calls below show a Perl command line instance and the
+equivilent calls to C<edit_file> and C<edit_file_lines>.
+
+	perl -0777 -pi -e 's/foo/bar/g' filename
+	use File::Slurp ;
+	edit_file { s/foo/bar/g } 'filename' ;
+	edit_file sub { s/foo/bar/g }, 'filename' ;
+	edit_file \&replace_foo, 'filename' ;
+	sub replace_foo { s/foo/bar/g }
+
+	perl -pi -e '$_ = '' if /foo/' filename
+	use File::Slurp ;
+	edit_file_lines { $_ = '' if /foo/ } 'filename' ;
+	edit_file_lines sub { $_ = '' if /foo/ }, 'filename' ;
+	edit_file \&delete_foo, 'filename' ;
+	sub delete_foo { $_ = '' if /foo/ }
+
 =head2 read_dir
 
 This sub reads all the file names from directory and returns them to
@@ -1059,6 +1185,15 @@ list of files.
 
 	my @all_files = read_dir( '/path/to/dir', keep_dot_dot => 1 ) ;
 
+=head3 prefix
+
+If this boolean option is set, the string "$dir/" is prefixed to each
+dir entry. This means you can directly use the results to open
+files. A common newbie mistake is not putting the directory in front
+of entries when opening themn.
+
+	my @paths = read_dir( '/path/to/dir', prefix => 1 ) ;
+
 =head2 EXPORT
 
   read_file write_file overwrite_file append_file read_dir
